/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "rfmeter.h"

// ------------------------------------------------------------- PRIVATE MACROS 

#define RFMETER_DUMMY 0

// -------------------------------------------------------------- PRIVATE TYPES


// ------------------------------------------------------------------ CONSTANTS


// ------------------------------------------------------------------ VARIABLES


// ---------------------------------------------- PRIVATE FUNCTION DECLARATIONS 

static void rfmeter_generic_transfer ( rfmeter_t *ctx, 
                                       spi_master_transfer_data_t *block );

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void rfmeter_cfg_setup ( rfmeter_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->sck = HAL_PIN_NC;
    cfg->miso = HAL_PIN_NC;
    cfg->mosi = HAL_PIN_NC;
    cfg->cs = HAL_PIN_NC;

    cfg->spi_speed = SPI_MEDIUM; 
    cfg->spi_mode = SPI_MODE_0;
}

RFMETER_RETVAL rfmeter_init ( rfmeter_t *ctx, rfmeter_cfg_t *cfg )
{
    spi_master_config_t spi_cfg;

    spi_master_configure_default( &spi_cfg );
    spi_cfg.mode        = cfg->spi_mode;
    spi_cfg.speed       = cfg->spi_speed;
    spi_cfg.pins.sck    = cfg->sck;
    spi_cfg.pins.miso   = cfg->miso;
    spi_cfg.pins.mosi   = cfg->mosi;
    spi_cfg.chip_select = cfg->cs;

    if (  spi_master_open( &ctx->spi, &spi_cfg ) != SPI_SUCCESS )
    {
        return RFMETER_INIT_ERROR;
    }

    spi_master_set_dummy_data( &ctx->spi, RFMETER_DUMMY );

    return RFMETER_OK;

}

uint16_t rfmeter_read_data ( rfmeter_t *ctx )
{
    spi_master_transfer_data_t block;
    uint8_t rx_buf[ 2 ];
	uint16_t result;

    block.tx_length = 0;
    block.rx_buffer = rx_buf;
    block.rx_length = 2;

    rfmeter_generic_transfer( ctx, &block );
    
	result = rx_buf[ 0 ];
    result <<= 8;
    result |= rx_buf[ 1 ];
	
    return result;
}

uint16_t rfmeter_get_raw_data ( rfmeter_t *ctx )
{
    uint16_t result = 0;

    result = rfmeter_read_data( ctx );
    result &= RFMETER_FILTER_USEFULL_DATA;
    result >>= 1;

    return result;
}

float rfmeter_get_voltage ( rfmeter_t *ctx )
{
    float result;
    uint16_t reading;

    reading = rfmeter_get_raw_data( ctx );
    
    result = ( float ) reading;
    result *= RFMETER_DEF_VREF;
    result /= RFMETER_ADC_RESOLUTION;

    return result;
}

float rfmeter_get_signal_strenght ( rfmeter_t *ctx, float slope, 
                                    float intercept )
{
    float result;
    float voltage;

    voltage = rfmeter_get_voltage( ctx );

    if ( voltage > RFMETER_DEF_LIMIT_HIGH )
    {
        result = ( RFMETER_DEF_LIMIT_HIGH / slope ) + intercept;
    }

    else if ( voltage < RFMETER_DEF_LIMIT_LOW )
    {
        result = ( RFMETER_DEF_LIMIT_LOW / slope ) + intercept;
    }
    else
    {
        result = ( voltage / slope ) + intercept;
    }

    return result;
}

// ----------------------------------------------- PRIVATE FUNCTION DEFINITIONS

static void rfmeter_generic_transfer ( rfmeter_t *ctx, 
                                       spi_master_transfer_data_t *block )
{
    spi_master_start(&ctx->spi);
    spi_master_transfer(&ctx->spi, block);
    spi_master_stop(&ctx->spi);    
}

// ------------------------------------------------------------------------- END

